const SimpleDB = require('../dynamodb');
const chai = require('chai');
const chaiAsPromised = require('chai-as-promised');
chai.use(chaiAsPromised);
const expect = chai.expect;
chai.should();

const AWS = require('aws-sdk');

const config = {
    region: 'local',
    endpoint: 'http://127.0.0.1:8000',
    accessKeyId: 'not-important',
    secretAccessKey: 'no-important'
}

const TABLE_NAME = 'MyScanTestTable'

let params = {
    TableName: TABLE_NAME,
    KeySchema: [
        {
            AttributeName: 'userId',
            KeyType: 'HASH'
        },

        {
            AttributeName: 'ts',
            KeyType: 'RANGE'
        },

    ],
    AttributeDefinitions: [
        {
            AttributeName: 'userId',
            AttributeType: 'S'
        },

        {
            AttributeName: 'ts',
            AttributeType: 'S'
        },

    ],
    ProvisionedThroughput:  {
        ReadCapacityUnits: 1,
        WriteCapacityUnits: 1
    }
}

let timestampValue = '2018-04-26T11:51:14.299Z';
let timestampValue2 = '2018-04-26T11:51:15.299Z';
let timestampValue3 = '2018-04-26T11:51:16.299Z';
let timestampValue4 = '2018-04-26T11:51:17.299Z';
let timestampValue5 = '2018-04-26T11:51:18.299Z';


function createTable() {
    let dynamodb = new AWS.DynamoDB(config);

    return new Promise((resolve, reject) => {
        dynamodb.createTable(params, function (err, data) {
            if (err) {
                reject(err);
                return;
            }
 
            resolve(data);
        });
    })
}

function deleteTable() {
    let dynamodb = new AWS.DynamoDB(config);

    return new Promise((resolve, reject) => {
        dynamodb.deleteTable({ TableName: TABLE_NAME }, (err, data) => {
            if (err) {
                reject(err);
                return;
            }

            resolve(data);
        })
    })
}

function configureDocClient() {
    AWS.config.update(config);
    let dynamodb = new AWS.DynamoDB();
    let docClient = new AWS.DynamoDB.DocumentClient({
        options: dynamodb,
    });

    return docClient;
}

function prepopulateTable(docClient, item) {
    return new Promise((resolve, reject) => {
        docClient.put({
            TableName: TABLE_NAME,
            Item: item
        }, (err, data) => {
            if (err) {
                reject(err);
                return;
            }

            resolve();
        })
    })
}

function populateFewItems(docClient) {
    let p1 = prepopulateTable(docClient,{ userId: '123', ts: timestampValue });
    let p2 = prepopulateTable(docClient,{ userId: '123', ts: timestampValue2 });
    let p3 = prepopulateTable(docClient,{ userId: '123', ts: timestampValue3 });
    let p4 = prepopulateTable(docClient,{ userId: '123', ts: timestampValue4 });
    let p5 = prepopulateTable(docClient,{ userId: '123', ts: timestampValue5 });

    return Promise.all([p1, p2, p3, p4, p5]);
}

before(() => {
    let docClient = configureDocClient();

    return createTable().then(() => {
        return populateFewItems(docClient);
    })
        .catch(err => {
            configureDocClient();
            return populateFewItems(docClient);
        });
});

after(() => {
    return deleteTable();
});

describe('6 DynamoDB Scan Operation', () => {
    describe('6.1 Query Test', () => {
        it('6.1.1 Primary Key', () => {
            let db = new SimpleDB(config);
            return db.scan()
                .tableName(TABLE_NAME)    
                .limit(3)
                .run()    
                .then(result => {
                    expect(result.Items.length).to.be.equal(3)
                    expect(result.hasNextPage()).to.be.true;
                });
        })
    });
});