const SimpleDB = require('../dynamodb');
const chai = require('chai');
const chaiAsPromised = require('chai-as-promised');
chai.use(chaiAsPromised);
const expect = chai.expect;
chai.should();

const AWS = require('aws-sdk');

const config = {
    region: 'local',
    endpoint: 'http://127.0.0.1:8000',
    accessKeyId: 'not-important',
    secretAccessKey: 'no-important'
}

const TABLE_NAME = 'MyBatchWriteTestTable1'
const TABLE_NAME2 = 'MyBatchWriteTestTable2'

let params = {
    TableName: TABLE_NAME,
    KeySchema: [
        {
            AttributeName: 'userId',
            KeyType: 'HASH'
        },

        // {
        //     AttributeName: 'ts',
        //     KeyType: 'RANGE'
        // },

    ],
    AttributeDefinitions: [
        {
            AttributeName: 'userId',
            AttributeType: 'S'
        },

        // {
        //     AttributeName: 'ts',
        //     AttributeType: 'S'
        // },

    ],
    ProvisionedThroughput:  {
        ReadCapacityUnits: 10,
        WriteCapacityUnits: 10
    }
}

let params2 = {
    TableName: TABLE_NAME2,
    KeySchema: [
        {
            AttributeName: 'userId',
            KeyType: 'HASH'
        },

        // {
        //     AttributeName: 'ts',
        //     KeyType: 'RANGE'
        // },

    ],
    AttributeDefinitions: [
        {
            AttributeName: 'userId',
            AttributeType: 'S'
        },

        // {
        //     AttributeName: 'ts',
        //     AttributeType: 'S'
        // },

    ],
    ProvisionedThroughput:  {
        ReadCapacityUnits: 1,
        WriteCapacityUnits: 1
    }
}

let timestampValue = '2018-04-26T11:51:14.299Z';
let timestampValue2 = '2018-04-26T11:51:15.299Z';
let timestampValue3 = '2018-04-26T11:51:16.299Z';
let timestampValue4 = '2018-04-26T11:51:17.299Z';
let timestampValue5 = '2018-04-26T11:51:18.299Z';


function createTable() {
    let dynamodb = new AWS.DynamoDB(config);

    let p1 = new Promise((resolve, reject) => {
        dynamodb.createTable(params, function (err, data) {
            if (err) {
                reject(err);
                return;
            }
 
            resolve(data);
        });
    })

    let p2 = new Promise((resolve, reject) => {
        dynamodb.createTable(params2, function (err, data) {
            if (err) {
                reject(err);
                return;
            }
 
            resolve(data);
        });
    })

    return Promise.all([p1, p2]);
}

function deleteTable() {
    let dynamodb = new AWS.DynamoDB(config);

    let p1 = new Promise((resolve, reject) => {
        dynamodb.deleteTable({ TableName: TABLE_NAME }, (err, data) => {
            if (err) {
                reject(err);
                return;
            }

            resolve(data);
        })
    })

    let p2 = new Promise((resolve, reject) => {
        dynamodb.deleteTable({ TableName: TABLE_NAME2 }, (err, data) => {
            if (err) {
                reject(err);
                return;
            }

            resolve(data);
        })
    })

    return Promise.all([p1, p2]);
}

function configureDocClient() {
    AWS.config.update(config);
    let dynamodb = new AWS.DynamoDB();
    let docClient = new AWS.DynamoDB.DocumentClient({
        options: dynamodb,
    });

    return docClient;
}

function prepopulateTable(docClient, table, item) {
    return new Promise((resolve, reject) => {
        docClient.put({
            TableName: table,
            Item: item
        }, (err, data) => {
            if (err) {
                reject(err);
                return;
            }

            resolve();
        })
    })
}

function populateFewItems(docClient) {
    let p1 = prepopulateTable(docClient, TABLE_NAME, { userId: 'aaa', ts: timestampValue });
    let p2 = prepopulateTable(docClient, TABLE_NAME, { userId: 'bbb', ts: timestampValue2 });
    let p3 = prepopulateTable(docClient, TABLE_NAME2, { userId: 'ccc', ts: timestampValue3 });
    let p4 = prepopulateTable(docClient, TABLE_NAME2, { userId: 'ddd', ts: timestampValue4 });
    // let p5 = prepopulateTable(docClient, TABLE_NAME, { userId: '123', ts: timestampValue5 });

    return Promise.all([p1, p2, p3, p4]);
}

before(() => {
    let docClient = configureDocClient();

    return createTable()
        .then(() => {
            return populateFewItems(docClient);
        })
        .catch(err => {
            return populateFewItems(docClient);
        });
});

after(() => {
    return deleteTable();
});

describe('9 DynamoDB Batch Write Operation', () => {
    it('9.1 Basic', () => {
        let db = new SimpleDB(config);
        return db.batchWrite()
            .tableName(TABLE_NAME)
            .delete().primaryKey('userId').equal('aaa')
            .delete().primaryKey('userId').equal('bbb')
            .put().primaryKey('userId').equal('ccc')
            .attribute('ts').equal(timestampValue3)
            .put().primaryKey('userId').equal('ddd')
            .attribute('ts').equal(timestampValue4)
            .tableName(TABLE_NAME2)
            .delete().primaryKey('userId').equal('ccc')
            .delete().primaryKey('userId').equal('ddd')
            .put().primaryKey('userId').equal('ggg')
            .attribute('ts').equal(timestampValue)
            .put().primaryKey('userId').equal('hhh')
            .attribute('ts').equal(timestampValue2)
            .returnedConsumedCapacity('TOTAL')
            .returnItemCollectionMetricsSize()
            .run();
           
    });

    it('9.2 Array', () => {
        let db = new SimpleDB({ dummy: 'dummy' });
        return db.batchWrite()
            .tableName(TABLE_NAME)
            .delete([{ userId: 'ccc' }, { userId: 'ddd' }])
            .put([{ userId: 'aaa', name: 'Alice' }, { userId: 'bbb', name: 'Bob' }])
            .returnedConsumedCapacity('TOTAL')
            .returnItemCollectionMetricsSize()
            .run()
    });

    it('9.3 With Request Items', () => {
        let db = new SimpleDB({ dummy: 'dummy' });
        return db.batchWrite()
            .withRequestItems({
                MyBatchWriteTestTable1: [
                    { DeleteRequest: { Key: { userId: 'aaa' } } },
                    { DeleteRequest: { Key: { userId: 'bbb' } } },
                ]
            })
            .returnedConsumedCapacity('TOTAL')
            .returnItemCollectionMetricsSize()
            .run()

    });
});